﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PDBInfoFromPE
{
    public class CV_HEADER
    {
        public string CvSignature;
        public int Offset;

        public static string ReadString(BinaryReader reader)
        {
            List<byte> buf = new List<byte>();
            while (true)
            {
                byte ch = reader.ReadByte();
                if (ch == 0)
                {
                    break;
                }

                buf.Add(ch);
            }

            return Encoding.UTF8.GetString(buf.ToArray());
        }
    }
    public class CV_INFO_PDB20 : CV_HEADER
    {
        public uint Signature;
        public uint Age;
        public string PdbFileName;

        public static CV_INFO_PDB20 FromReader(BinaryReader reader)
        {
            CV_INFO_PDB20 info = new CV_INFO_PDB20();
            
            info.CvSignature = "NB10";
            info.Offset = reader.ReadInt32();

            Debug.Assert(info.Offset == 0);

            info.Signature = reader.ReadUInt32();
            info.Age = reader.ReadUInt32();
            info.PdbFileName = CV_HEADER.ReadString(reader);

            return info;
        }

        public override string ToString()
        {
            return string.Format("Signature: {0}, Age: {1}, Path: {2}", Signature, Age, PdbFileName);
        }

        public string GetDownloadPath()
        {
            string fileName = Path.GetFileName(PdbFileName);
            return string.Format("\\{0}\\{1}{2}\\{0}", fileName, Signature, Age);
        }
    }

    public class CV_INFO_PDB70
    {
        public string CvSignature;
        public Guid Signature;
        public uint Age;
        public string PdbFileName;

        public static CV_INFO_PDB70 FromReader(BinaryReader reader)
        {
            CV_INFO_PDB70 info = new CV_INFO_PDB70();

            info.CvSignature = "RSDS";
            info.Signature = new Guid(reader.ReadBytes(16));
            info.Age = reader.ReadUInt32();
            info.PdbFileName = CV_HEADER.ReadString(reader);

            return info;
        }

        public override string ToString()
        {
            return string.Format("Signature: {0}, Age: {1}, Path: {2}", Signature, Age, PdbFileName);
        }

        public string GetDownloadPath()
        {
            string fileName = Path.GetFileName(PdbFileName);
            return string.Format("\\{0}\\{1:N}{2}\\{0}", fileName, Signature, Age);
        }
    }
}
