﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Workshell.PE;
using Workshell.PE.Debug;

namespace PDBInfoFromPE
{
    class Program
    {
        static void Main(string[] args)
        {
            Dump("ConsoleApp1.exe");
            Dump("PDBInfoFromPE.exe");
            Dump("msvcirt.dll");
        }

        private static void Dump(string filePath)
        {
            Workshell.PE.ExecutableImage pe = Workshell.PE.ExecutableImage.FromFile(filePath);

            Console.WriteLine("PE File: " + filePath);

            DebugDirectory debugDir = DebugDirectory.Get(pe);

            if (debugDir == null)
            {
                Console.WriteLine("No IMAGE_DEBUG_DIRECTORY");
            }
            else
            {
                foreach (var debugEntry in debugDir)
                {
                    if (debugEntry.Type == 2) // 2 == IMAGE_DEBUG_TYPE_CODEVIEW
                    {
                        ShowDebugData(debugEntry);
                    }
                }
            }

            Console.WriteLine();
        }

        private static void ShowDebugData(DebugDirectoryEntry debugEntry)
        {
            DebugData debugData = DebugData.Get(debugEntry);
            byte [] buf = debugData.GetBytes();

            DumpCodeViewDebugInfo(buf);
        }

        private static void DumpCodeViewDebugInfo(byte[] buf)
        {
            BinaryReader reader = new BinaryReader(new MemoryStream(buf), Encoding.Default);
            string cvSignature = Encoding.ASCII.GetString(reader.ReadBytes(4));
            Console.WriteLine("CodeView Info Type: " + cvSignature);

            switch (cvSignature)
            {
                case "NB10":
                    DumpNB10CodeViewInfo(reader);
                    break;
                case "RSDS":
                    DumpRSDSCodeViewInfo(reader);
                    break;

                default:
                    Console.WriteLine("NOT supported debug info: " + cvSignature);
                    break;
            }
        }

        private static void DumpNB10CodeViewInfo(BinaryReader reader)
        {
            CV_INFO_PDB20 info = CV_INFO_PDB20.FromReader(reader);
            Console.WriteLine(info);
            Console.WriteLine("PDB Download Path: " + info.GetDownloadPath());
        }

        private static void DumpRSDSCodeViewInfo(BinaryReader reader)
        {
            CV_INFO_PDB70 info = CV_INFO_PDB70.FromReader(reader);
            Console.WriteLine(info);
            Console.WriteLine("PDB Download Path: " + info.GetDownloadPath());
        }

        /*
        private static void ShowDebugDirectoryInfo(Workshell.PE.ExecutableImage pe)
        {
            foreach (var item in pe.NTHeaders.DataDirectories)
            {
                if (item.DirectoryType == Workshell.PE.DataDirectoryType.Debug)
                {
                    Console.WriteLine("Debug directory: ");
                    Console.WriteLine("\t RVA: " + item.VirtualAddress.ToString("x"));
                    Console.WriteLine("\t Size: " + item.Size.ToString("x"));

                    LocationCalculator calc = pe.GetCalculator();
                    ulong offset = calc.RVAToOffset(item.VirtualAddress);
                    Console.WriteLine(offset);

                    // offset == IMAGE_DEBUG_TYPE_CODEVIEW 영역을 가리키는 파일의 위치
                    // offset = item.VirtualAddress - textSection.Location.RelativeVirtualAddress + textSection.Location.FileOffset;

                    return;
                }
            }
        }
        */
    }
}
